
/**********************************************************************
* ?2006 Microchip Technology Inc.
*
* FileName:        main_rtc.c
* Dependencies:    p33FJ256GP710.h
* Processor:       dsPIC33F
* Compiler:        MPLAB?C30 v2.01 or higher
*
* SOFTWARE LICENSE AGREEMENT:
* Microchip Technology Inc. (Microchip? licenses this software to you
* solely for use with Microchip dsPIC?digital signal controller
* products. The software is owned by Microchip and is protected under
* applicable copyright laws.  All rights reserved.
*
* SOFTWARE IS PROVIDED AS IS.? MICROCHIP EXPRESSLY DISCLAIMS ANY
* WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
* PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP
* BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
*
* REVISION HISTORY:
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Author            Date      Comments on this revision
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Richard Fischer   07/14/05  Initial Release
* Priyabrata Sinha  01/27/06  Ported to non-prototype devices 
* Priyabrata Sinha  01/30/06  Added config macros
*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
* ADDITIONAL NOTES:  
*
* Simple demo program showing a RTC and ADC conversion on the LCD
* DMA code is included here but not fully tested.
*
* C30 Optimization Level: -O1
*
**********************************************************************/

#include 	"p33Fxxxx.h"
#include 	"LCD_APP026_2.h"
#include 	<math.h>
#include	<stdio.h>

// _FGS(GWRP_OFF & GCP_OFF);
// _FOSCSEL(FNOSC_PRIPLL);
// _FOSC(FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMD_XT);
// _FWDT(FWDTEN_OFF);

void	ADC_Initial(void);
void	Initial_Timer1( void );
void 	MotPWM_Initial(void);
void	DMA_Initial4AD(void);
void	My_itoa(unsigned int , unsigned char *);

const char mytext[] =  "  APP026-2 EVM  ";
const char mytext1[] = "A1=      A2=    ";

unsigned int DMA_ADBufA[32] __attribute__((space(dma)));
unsigned int DMA_ADBufB[32] __attribute__((space(dma)));

#define	INPUT	1
#define	OUTPUT	0

#define		Ctrl_U1RX 	TRISFbits.TRISF2 
#define		Ctrl_U1TX 	TRISFbits.TRISF3 

#define	LED0	LATDbits.LATD0 
#define	LED1	LATDbits.LATD1
#define	LED2	LATDbits.LATD2
#define	LED3	LATDbits.LATD3

union	
	{
		struct	{
					unsigned char	ByteL ;
					unsigned char	ByteH ;

				};
					unsigned int	WordVal ;

	}Byte2Word ;

int			VR1_Value , VR2_Value ;
unsigned	ValuePDC ;
int			ConvertOK = 0 ;
int			T1Overflow = 0 ;
int			T1Counter = 0 ;

unsigned char ASCII_Buffer[10] ;


// Using internal FRC oscillator at 8MHz
// Fosc = Fin * (DIVISOR<8:0> + 2 / ((PRESCLR<4:0> +2) * (PLLPOST<1:0> +1) )
// DIVISOR<8:0> bits located in PLLFBF SFR ---> (PLL Multiplier)
// PLLPOST<1:0> bits located in CLKDIV SFR ---> (PLL VCO output divide)
// PRESCLR<4:0> bits located in CLKDIV SFR ---> (PLL Phase Detector input divide)

 
void 	_ISR _T1Interrupt(void)
{
		IFS0bits.T1IF = 0 ;
		T1Counter ++ ;
		if (T1Counter >= 200)
		{
			T1Counter = 0 ;
			T1Overflow = 1 ;

			LED0 = !LED0 ;
			LED1 = !LED1 ;
		}
}

void	_ISR	_DMA0Interrupt(void)
{
		VR1_Value = DMA_ADBufA[16];
		VR2_Value = DMA_ADBufA[20];
		IFS0bits.DMA0IF = 0 ;
		ConvertOK = 1 ;
}

int main ( void )
{

        //The settings below set up the oscillator and PLL for 16 MIPS as
        //follows:
        //            Crystal Frequency  * (DIVISOR+2)
        // Fcy =     ---------------------------------
        //              PLLPOST * (PRESCLR+2) * 4	
 		//       PLLFBD = 0x00A0;
 		//       CLKDIV = 0x0048;
 
	TRISDbits.TRISD0 = OUTPUT ;
	TRISDbits.TRISD1 = OUTPUT ;
	TRISDbits.TRISD2 = OUTPUT ;
	TRISDbits.TRISD3 = OUTPUT ;

	Initial_Timer1();
	ADC_Initial();
	DMA_Initial4AD();
	UART1_Initial() ;

	/* Initialize LCD Display */
	Initial_LCD();

	/* Welcome message */
	home_clr();
	putsLCD( (char*) &mytext[0]);
	line_2();
	putsLCD( (char*) &mytext1[0]);
	
	ConvertOK = 0 ;	
 
    while ( 1 ) 
    { 

		if (ConvertOK && T1Overflow)
			{
				ConvertOK = 0 ;
				T1Overflow = 0 ;

				My_itoa(VR1_Value , ASCII_Buffer) ;
				setcursorLCD(3,1) ;
				putrsLCD("    ") ;
				setcursorLCD(3,1) ;
				putsLCD(ASCII_Buffer) ;
				My_itoa(VR2_Value , ASCII_Buffer) ;
				setcursorLCD(12,1) ;
				putrsLCD("    ") ;
				setcursorLCD(12,1) ;
				putrsLCD(ASCII_Buffer) ;

				printf ( " Hello ! ڬO dsPIC33FJ128MC708 \n\r" ) ;
				
			}
	}
 
}

void	UART1_Initial(void)
{

		U1MODE = 0x8800 ;
		U1STA = 0x2400 ;
		U1BRG = 51 ;				// Baud-Rate = 4800 when Fcy = 4M , BRGH = 0 

		Ctrl_U1RX = INPUT ;
		Ctrl_U1TX = OUTPUT ;

}

void	Initial_Timer1( void )
{
		T1CON = 0x0000 ;
		PR1 = 4096 ;
		IPC0bits.T1IP = 4 ;
		IFS0bits.T1IF = 0 ;
		IEC0bits.T1IE = 1 ;
		T1CONbits.TON = 1 ;

//		ConfigIntTimer1( T1_INT_PRIOR_7 & T1_INT_ON ) ;
//		OpenTimer1( T1_ON & T1_IDLE_STOP & T1_GATE_OFF & T1_PS_1_1 & T1_SYNC_EXT_OFF & T1_SOURCE_INT ,
//					2048 ) ;
}

void 	MotPWM_Initial(void)
{
		IEC3bits.PWMIE = 0 ;		// Disable PWM Interrupt !!
		IEC3bits.FLTAIE = 0 ;	

		OVDCON = 0xff00 ;			// Inactive all PWM OUTPUT !!

		TRISE = 0xffc0 ;		
		PTCON = 0xa000 ;		// Configure as 0b1010 0000 0000 0000 
								// PWM Time Base OFF , PWM Time Base OP in free running Mode 
		PWMCON1 = 0x0077 ;		// Configure as 0b0000000000010001
								// PWM I/O in complementary Mode and only PWM1L/H as PWM output 
		PWMCON2 = 0x0000 ;		// Configure as 0b0000000000000000 

		DTCON1 = 0x0000 ;		// Configure as 0b0000001000000010 ;

		FLTACON = 0x0000 ;

		IPC14bits.PWMIP = 6 ;

		// ---------------------------------------------------------------------------------
		// The Switching Frequency !!
		// PWM resolution >= 10 bits , 
		// PDCx[1:15] compare with PTMR [0:14]
		// PDCx(0) compare with MSB of prescaler counter
		// So, PTPER is 9 bit if resolution of PDCx is 10 bit
		// Setting PWM Frequency = 20K
		// PTPER = ( (5000000)/ 20000 ) -1 = 249
		// PWM Frequency will be Fcy/736 = 20.0K
		// Formular !! PTPER = (Fcy/(FPWM*PTMR Prescaler)) - 1 
		// ---------------------------------------------------------------------------------
		PTPER = 249 ;			// PWM Time Base Period Register 
		ValuePDC = 249 ;

			PDC1 = ValuePDC ;
			PDC2 = ValuePDC ;
			PDC3 = ValuePDC ;		

}

void	DMA_Initial4AD(void)
{

	DMACS0 = 0x0000 ;
	DMA0CON = 0b0000000000100010 ;
	DMA0REQ = 0x000d ;
	DMA0STA = 	__builtin_dmaoffset(DMA_ADBufA);
	DMA0STB = 	__builtin_dmaoffset(DMA_ADBufB);
	DMA0PAD = ( volatile unsigned int ) & ADC1BUF0 ;
	DMA0CNT = 0x0007 ;
	IFS0bits.DMA0IF = 0 ;
	IEC0bits.DMA0IE = 1 ;
	DMA0CONbits.CHEN = 1 ;

}

void	ADC_Initial(void)
{
	AD1CON1 = 0x20E4 ;		// 0010 0000 1110 0100
	AD1CON2 = 0x0404 ;		// 0000 0100 0000 0100
	AD1CON3 = 0x9008 ;
	AD1CON4 = 0x0002 ;		// 0000 0000 0000 0010
	AD1CHS123 = 0x0000 ;
	AD1CHS0 =  0x0004 ;

	AD1CSSL = 0x0000 ;
	AD1CSSLbits.CSS4 = 1 ;
	AD1CSSLbits.CSS5 = 1 ;

	AD1PCFGL = 0b1111111111001111 ;

	AD1PCFGLbits.PCFG4 = 0 ;			// RB4 as Analog input ::
	AD1PCFGLbits.PCFG5 = 0 ;			// RB5 as Analog input ::
	AD1CON1bits.ADON = 1 ;
	IFS0bits.AD1IF = 0 ;

}


void	My_itoa(unsigned int IntValue , unsigned char *TheString)
{

		int	TempValue1 , TempValue2 , TempValue3 , TempValue4 , TempValue5 ;

		int	ZeroFill ;

		ZeroFill = 0 ;		

		TempValue1 = IntValue/10000 ;
		TempValue2 = (IntValue-(TempValue1*10000))/1000 ;
		TempValue3 = (IntValue-(TempValue1*10000)-(TempValue2*1000))/100 ;
		TempValue4 = (IntValue-(TempValue1*10000)-(TempValue2*1000)- (TempValue3*100))/10 ;
		TempValue5 = IntValue % 10 ;

		if (TempValue1 != 0 )
			{
				*TheString++ = TempValue1 + '0' ;
				ZeroFill = 1 ;
			}
		if (TempValue2 != 0 || ZeroFill  )
			{
				*TheString++ = TempValue2 + '0' ;
				ZeroFill = 1 ;
			}
		if (TempValue3 != 0 || ZeroFill ) 
			{
				*TheString++ = TempValue3 + '0' ;
				ZeroFill = 1 ;
			}
		if (TempValue4 != 0 || ZeroFill )
			{
				*TheString++ = TempValue4 + '0' ;
				ZeroFill = 1 ;
			}
			*TheString++ = TempValue5 + '0' ;
			*TheString = 0x00 ;				//	End of string !!	

}
